<?php
/*
 * This class implements all the features of Comapping Server API
 */
class ComapApi {
	// comapping server url
	public $server = null;
	// user login
	public $username = null;
	// user password
	public $password = null;
	// session id
	public $sessionId = null;
	
	// xml_handlers for response (should be defined)
	public $xml_start_element_handler = null;
	public $xml_character_data_handler = null;
	
	function ComapApi($username, $password, $server = null) {
		$this->server = (is_null($server)) ? "http://go.comapping.com/cgi-bin/comapping.n" : $server;
		$this->username = $username;
		$this->password = $password;
	}
	
	// opens a user session
	public function openSession() { 
		$login_request_params = array();
		$login_request_params['action'] = 'login';
		$login_request_params['loginMethod'] = 'simple';
		$login_request_params['login'] = $this->username;
		$login_request_params['password'] = md5($this->password);
		
		$this->makeLoginRequest($login_request_params);		
	}
	
	// makes a login request
	public function makeLoginRequest($params) {
		$result = $this->request($params);
		$this->processXMLResponse($result);
	}
	
	private function processXMLResponse($result) {
		$xml_parser = xml_parser_create();
		xml_set_element_handler($xml_parser, $this->xml_start_element_handler, null);
		xml_set_character_data_handler($xml_parser, $this->xml_character_data_handler);
		xml_parse($xml_parser, $result);	
		xml_parser_free($xml_parser);	
	}
	
	// downloads the map in specified fromat and saves it on the disk
	public function saveMap($map_id, $format, $filename) {
		$params = array();
		$params['action'] = 'download';
		$params['clientID'] = $this->sessionId;
		$params['mapid'] = $map_id;
		$params['format'] = $format;
		
		$file = fopen($filename, "w");
		fwrite($file, $this->request($params));
	}
	
	// moves topic to a new parent
	public function moveTopic($topic_id, $newParent, $index, $map_id) {
		$this->sendCommand("<commands mapid=\"" . $map_id . "\"><moveTopic id=\"" . $topic_id . "\" newparentid=\"" . $newParent . "\" index=\"" . $index . "\"/></commands>");
	}
	
	// adds a subtopic
	public function addTopic($parent, $child_data, $map_id) {
		$this->sendCommand("<commands mapid=\"" . $map_id . "\"><addTopic parentid=\"" . $parent . "\">" . $child_data . "</addTopic></commands>");
	}
	
	// removes topic
	public function removeTopic($topic_id, $map_id) {
		$this->sendCommand("<commands mapid=\"" . $map_id . "\"><removeTopic childid=\"" . $topic_id . "\"/></commands>");
	}
	
	// removes all subtopics of the topic
	public function removeSubtopics($topic_id, $map_id) {
		$this->sendCommand("<commands mapid=\"" . $map_id . "\"><removeSubtopics id=\"" . $topic_id . "\"/></commands>");
	}
	
	// replaces topic content with a new one
	public function changeTopic($topic_id, $new_data, $map_id) {
		$this->sendCommand("<commands mapid=\"" . $map_id . "\"><changeTopic id=\"" . $topic_id . "\">" . $new_data . "</changeTopic></commands>");
	}		
	
	// change permissions of the map
	public function changePermissions($users, $message, $map_id) {
		$command = "<commands mapid=\"" . $map_id . "\"><changePermissions message=\"" . $message . "\">";
		foreach ($users as $user) {
			$command .= "<user name=\"" . $user->name . "\" email=\"" . $user->email . "\" access=\"" . $user->access . "\"/>";
		}
		$command .= "</changePermissions></commands>";
		$this->sendCommand($command);
	}
	
	private function sendCommand($command) {
		$params = array();
		$params['action'] = 'broadcast_commands';
		$params['clientID'] = $this->sessionId;
		$params['data'] = $command;
		
		$this->processXMLResponse($this->request($params));
	}
		
	private function request($params) {
		if (empty($this->server)) {
			return false;
		}

		$url_parsed = parse_url($this->server);

		$host = $url_parsed["host"];
		$port = $url_parsed["port"];
		
		if ( $port == 0 ) {
			$port = 80;
		}
		$path = $url_parsed["path"];
		
		if ( empty( $path ) ) {
			$path="/";
		}

		if ( $url_parsed["query"] != "" ) {
			$path .= "?";
			// URL-encode the query (encode '@', ' ', etc)
			$query_str = $url_parsed["query"];
			$query_params = explode('&', $query_str);
			foreach ($query_params as $param) {
				 list($name, $value) = explode('=', $param, 2);
				 $params[$name] = $value;
			}
		}
		
		$path .= ("?" . http_build_query($params));
		
		$out = "GET $path HTTP/1.0\r\nHost: $host\r\n\r\n";
		$fp = fsockopen( $host, $port, $errno, $errstr, 30 );

		if ( $fp ) {
			fwrite( $fp, $out );

			$return = '';
		
			while ( !feof( $fp ) ) {
				$return .= fgets($fp, 1024);
			}
			fclose( $fp );
			return $this->getXmlResponse($return);
		} else {
			return null;
		}		
	}
	
	private function getXmlResponse($response) {
		$content = '';
		$str = strtok($response, "\n");
		$h = null;
		
		while ($str !== false) {
			if ($h and trim($str) === '') {
				$h = false;
				continue;
			}
			if ($h !== false and false !== strpos($str, ':')) {
				// skip headers
				$h = true;
			}
			if ($h === false) {
				$content .= $str."\n";
			}
			$str = strtok("\n");
		}
		return trim($content);
	}
}

class UserInfo {
	public $name;
	public $email;
	public $access; // view, edit, none
	
	public function UserInfo($name, $email, $access) {
		$this->name = $name;
		$this->email = $email;
		$this->access = $access; 
	}
}
?>